// tempus 2000.11g
#ifndef TARGET_SYSTEM_CLASS
#define TARGET_SYSTEM_CLASS
//
// (c) Copyright 1995-2001, MZA Associates Corporation
//

#include "tempus.h"


#include "TargetBoard.h"
#include "FpaProcessing.h"
#include "PulsedPointSource.h"
#include "SquareWave.h"
#include "IncomingCombiner.h"
#include "MultiPointSource.h"
#include "TransverseVelocity.h"

class Target : public System
{
public:

   // Parameters
   double control_interval;
   float range;
   Vector<float> vtarg;
   float tbd_wavelength;
   int tbd_nxy;
   float tbd_dxy;
   float beacon_wavelength;
   float beacon_x;
   float beacon_y;
   float img_wavelength;
   Vector<float> img_x;
   Vector<float> img_y;

   // Subsystems
   TargetBoard targetboard;

   FpaProcessing targetboardproc;

   PulsedPointSource pointsource;

   SquareWave zoverc;

   IncomingCombiner combiner;

   MultiPointSource multipointsource;

   TransverseVelocity transversevelocity;


   // Inputs
   Input< WaveTrain > incident;

   // Outputs
   Output< WaveTrain > transmitted;

private:      // Setting expressions for unsatisfied inputs/external outputs
   Output< double >  targetboard_exposureInterval;
   Output< double >  targetboard_exposureLength;
   Output< double >  targetboard_sampleInterval;
   Output< bool >  pointsource_on;
   Output< bool >  multipointsource_on;

public:
   Target(SystemNode *parent, char *name,
          double _control_interval,
          float _range,
          Vector<float> _vtarg,
          float _tbd_wavelength,
          int _tbd_nxy,
          float _tbd_dxy,
          float _beacon_wavelength,
          float _beacon_x,
          float _beacon_y,
          float _img_wavelength,
          Vector<float> _img_x,
          Vector<float> _img_y) : 
      System(parent, name),
      control_interval(_control_interval),
      range(_range),
      vtarg(_vtarg),
      tbd_wavelength(_tbd_wavelength),
      tbd_nxy(_tbd_nxy),
      tbd_dxy(_tbd_dxy),
      beacon_wavelength(_beacon_wavelength),
      beacon_x(_beacon_x),
      beacon_y(_beacon_y),
      img_wavelength(_img_wavelength),
      img_x(_img_x),
      img_y(_img_y),
      targetboard(this, "targetboard",
         tbd_wavelength,
         tbd_nxy,
         tbd_dxy),
      targetboardproc(this, "targetboardproc",
         tbd_nxy,
         tbd_nxy,
         tbd_dxy,
         tbd_dxy,
         0.0,
         1,
         10),
      pointsource(this, "pointsource",
         control_interval,
         1.0e-06,
         1.0e+06,
         beacon_wavelength,
         beacon_x,
         beacon_y),
      zoverc(this, "zoverc",
         (range/speedOfLight),
         1.0,
         1.0e+06,
         1.0e+06),
      combiner(this, "combiner"),
      multipointsource(this, "multipointsource",
         control_interval,
         1.0e-06,
         1.0e+06,
         img_x,
         img_y,
         img_wavelength),
      transversevelocity(this, "transversevelocity",
         -vtarg(0),
         -vtarg(1),
         0.0,
         0.0),
      incident(this, "incident"),
      transmitted(this, "transmitted"),
      targetboard_exposureInterval(this, "targetboard_exposureInterval"),
      targetboard_exposureLength(this, "targetboard_exposureLength"),
      targetboard_sampleInterval(this, "targetboard_sampleInterval"),
      pointsource_on(this, "pointsource_on"),
      multipointsource_on(this, "multipointsource_on")
   {
      targetboardproc.image <<= targetboard.integrated_intensity;
      targetboard.on <<= zoverc.boolOutputSignal;
      combiner.incomingIncident <<= pointsource.transmitted;
      combiner.incomingIncident2 <<= multipointsource.transmitted;
      targetboard.incident <<= combiner.outgoingTransmitted;
      transversevelocity.outgoingIncident <<= incident;
      transversevelocity.incomingIncident <<= combiner.incomingTransmitted;
      transmitted <<= transversevelocity.incomingTransmitted;
      combiner.outgoingIncident <<= transversevelocity.outgoingTransmitted;

      // System's Outputs and Subsystems' Inputs that
      // are not connected, but have setting expressions
      targetboard.exposureInterval <<= targetboard_exposureInterval;
      targetboard.exposureLength <<= targetboard_exposureLength;
      targetboard.sampleInterval <<= targetboard_sampleInterval;
      pointsource.on <<= pointsource_on;
      multipointsource.on <<= multipointsource_on;
   }

   // void respondToScheduledEvent (const Event & /* event */);
   // void respondToChangedInputs();
   // void respondToInputWarning (const InputBase* /* input */);
   void respondToOutputRequest (const OutputBase* output)
   {
      double t = now();
      if(*output == targetboard_exposureInterval)
      {
         targetboard_exposureInterval = (double)(control_interval);
      }
      else if(*output == targetboard_exposureLength)
      {
         targetboard_exposureLength = (double)(1.0e-6);
      }
      else if(*output == targetboard_sampleInterval)
      {
         targetboard_sampleInterval = (double)(-1.0);
      }
      else if(*output == pointsource_on)
      {
         pointsource_on = (bool)(true);
      }
      else if(*output == multipointsource_on)
      {
         multipointsource_on = (bool)(true);
      }
   }
};

#endif // TARGET_SYSTEM_CLASS
